#!/bin/bash
# 编写时间：2021-09-12
# 编写人： LiuChangChun
# 版本号：1.0.2
# 修改时间：2021-10-09  更新了输入项的约束以及循环判断
echo "********************1.先决条件检查************************"
# 检查主机上是否已经安装了mariadb
if [ `rpm -qa | grep mysql | wc -l` -gt 0 ] || [ `systemctl list-unit-files | grep mariadb | grep -v grep | wc -l` -gt 0 ] || 
   [ `ps -ef | grep -w mariadb | grep -v grep  | wc -l` -gt 0 ] || [ `ps -ef | grep -w mariadbd | grep -v grep  | wc -l` -gt 0 ]; then
  echo -e "\033[31m 该主机已经安装了mariadb/mysql,请勿重复安装 \033[0m"
  exit 0
fi

# 检查/usr/local目录下是否存在mysql目录
if [ -d "/usr/local/mysql" ]; then
  echo -e "\033[31m /usr/local目录下存在mysql文件夹，请先删除 \033[0m"
  exit 0
fi

# 检查是否安装了libaio
if [ `rpm -qa | grep libaio | wc -l` -eq 0 ]; then
  echo -e "\033[31m 主机缺少libaio包，请配置yum源，使用 yum install libaio* -y 命令进行安装 \033[0m"
  exit 0
elif [ `cat /etc/redhat-release|sed -r 's/.* ([0-9]+)\..*/\1/'` -eq 8 ] && 
    ([ `rpm -qa | grep ncurses-compat-libs | wc -l` -eq 0 ] ||
     [ `rpm -qa | grep ncurses-c++-libs | wc -l` -eq 0 ]); then
  echo -e "\033[31m 主机缺少libncurses包，请配置yum源，使用 yum install libncurses* -y 命令进行安装 \033[0m"
  exit 0
fi

echo -e "\033[32m***********************SUCCESS**************************** \033[0m"
echo -e "\n"

while :
  do
    echo -e "\033[45;37m 请输入mariadb-10.5.8-linux-systemd-x86_64.tar.gz文件所在的目录,目录结尾不需要/ \033[0m"
    read softDir
    # 检查安装源是否存在
    if [ ! -f $softDir/mariadb-10.5.8-linux-systemd-x86_64.tar.gz ]; then
      echo -e "\033[31m mariadb-10.5.8-linux-systemd-x86_64.tar.gz 文件不存在，请重新输入 \033[0m"
    else
      break
    fi
  done

echo -e "\n"

while :
  do
    echo -e  "\033[45;37m 请输入MariaDB数据文件存储位置,目录结尾不需要/ \033[0m"
    read dataDir 
    # 检查安装目录是否存在
    if [ ! -d $dataDir ]; then
      echo -e "\033[31m $dataDir 目录不存在，请重新输入数据文件存储目录 \033[0m"
    elif [ "$(ls -A $dataDir)" ]; then
      # 检查存储目录是否为空
      echo -e "\033[31m $dataDir 存储目录不为空，请重新输入数据文件存储目录 \033[0m"
    else
      break
    fi
  done

echo -e "\n"
echo "********************2.解压安装包************************"
cd $softDir
tar -zxf mariadb-10.5.8-linux-systemd-x86_64.tar.gz -C /usr/local/
cd /usr/local/
rm -rf mysql
ln -s /usr/local/mariadb-10.5.8-linux-systemd-x86_64 /usr/local/mysql
echo -e "\033[32m ***********************SUCCESS**************************** \033[0m"
echo -e "\n"

echo "********************3.创建用户及目录**********************"
# 检查系统中是否存在mysql用户
id mysql >& /dev/null
if [ $? -ne 0 ]; then
  useradd -M -s /sbin/nologin  mysql
fi

mkdir -p $dataDir/{data,socket,log,tmp}
chown -R mysql:mysql /usr/local/mysql
chown -R mysql:mysql $dataDir
echo -e "\033[32m ***********************SUCCESS**************************** \033[m"
echo -e "\n"

echo "*******************4.创建/etc/my.cnf文件******************"
if [ ! -f /etc/my.cnf ]; then
  touch /etc/my.cnf
fi

cat /dev/null > /etc/my.cnf
echo "# mariadb10.5.8 configuration "  >> /etc/my.cnf
echo "[client] "  >> /etc/my.cnf
echo "port = 6006 "  >> /etc/my.cnf
echo "socket = ${dataDir}/socket/mysqld.sock "  >> /etc/my.cnf
echo "default-character-set = utf8mb4 "  >> /etc/my.cnf
echo "# The MySQL server "  >> /etc/my.cnf
echo "#---Basic---# "  >> /etc/my.cnf
echo "[mysqld] "  >> /etc/my.cnf
echo "read-only=0 "  >> /etc/my.cnf
echo "server-id = 600601 "  >> /etc/my.cnf
echo "report_host = 8.8.8.8 "  >> /etc/my.cnf
echo "report-port=6006   "  >> /etc/my.cnf
echo "extra_port = 6106 "  >> /etc/my.cnf
echo "extra_max_connections=3 "  >> /etc/my.cnf
echo "port = 6006 "  >> /etc/my.cnf
echo "user = mysql "  >> /etc/my.cnf
echo "basedir = /usr/local/mysql "  >> /etc/my.cnf
echo "datadir = ${dataDir}/data "  >> /etc/my.cnf
echo "tmpdir = ${dataDir}/tmp "  >> /etc/my.cnf
echo "socket = ${dataDir}/socket/mysqld.sock "  >> /etc/my.cnf
echo "log_bin = ${dataDir}/log/mysql-bin.log "  >> /etc/my.cnf
echo "pid-file = ${dataDir}/socket/mysql.pid "  >> /etc/my.cnf
echo "skip-external-locking "  >> /etc/my.cnf
echo "skip-name-resolve "  >> /etc/my.cnf
echo "lower_case_table_names = 1 "  >> /etc/my.cnf
echo "default-time_zone = '+8:00' "  >> /etc/my.cnf
echo "default-storage-engine = INNODB "  >> /etc/my.cnf
echo "character-set-server = utf8mb4 "  >> /etc/my.cnf
echo "connect_timeout = 10 "  >> /etc/my.cnf
echo "wait_timeout = 28800 "  >> /etc/my.cnf
echo "interactive_timeout = 28800 "  >> /etc/my.cnf
echo "back_log = 1024 "  >> /etc/my.cnf
echo "event_scheduler = OFF "  >> /etc/my.cnf
echo "open_files_limit = 65535 "  >> /etc/my.cnf
echo "sql_mode=STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION "  >> /etc/my.cnf
echo "thread_cache_size = 1024 "  >> /etc/my.cnf
echo "query_cache_type = 0 "  >> /etc/my.cnf
echo "query_cache_size = 0 "  >> /etc/my.cnf
echo "log_warnings = 1 "  >> /etc/my.cnf
echo "#---binlog---# "  >> /etc/my.cnf
echo "log-bin = ${dataDir}/log/mysql-bin "  >> /etc/my.cnf
echo "binlog_format = row "  >> /etc/my.cnf
echo "max_binlog_size = 1024M "  >> /etc/my.cnf
echo "binlog_cache_size = 24M "  >> /etc/my.cnf
echo "expire-logs-days = 7 "  >> /etc/my.cnf
echo "sync_binlog = 1 "  >> /etc/my.cnf
echo "log-slave-updates=1 "  >> /etc/my.cnf
echo "slave-net-timeout = 10 "  >> /etc/my.cnf
echo "skip-slave-start "  >> /etc/my.cnf
echo "relay_log_recovery = 1 "  >> /etc/my.cnf
echo "relay_log = mysqld-relay-bin "  >> /etc/my.cnf
echo "slow_query_log = 1 "  >> /etc/my.cnf
echo "slow_query_log_file = ${dataDir}/log/mysql-slow.log "  >> /etc/my.cnf
echo "long_query_time = 0.5 "  >> /etc/my.cnf
echo "#---error log---# "  >> /etc/my.cnf
echo "log-error = $dataDir/log/error.log "  >> /etc/my.cnf
echo "#---per_thread_buffers---# "  >> /etc/my.cnf
echo "max_connections=8000 "  >> /etc/my.cnf
echo "max_user_connections=5000 "  >> /etc/my.cnf
echo "max_connect_errors=1000000 "  >> /etc/my.cnf
echo "key_buffer_size = 64M "  >> /etc/my.cnf
echo "max_allowed_packet = 1024M "  >> /etc/my.cnf
echo "table_open_cache = 6144 "  >> /etc/my.cnf
echo "table_definition_cache = 4096 "  >> /etc/my.cnf
echo "sort_buffer_size = 2M "  >> /etc/my.cnf
echo "read_buffer_size = 2M "  >> /etc/my.cnf
echo "read_rnd_buffer_size = 2M "  >> /etc/my.cnf
echo "join_buffer_size = 8M  "  >> /etc/my.cnf
echo "tmp_table_size = 256M "  >> /etc/my.cnf
echo "max_heap_table_size = 256M "  >> /etc/my.cnf
echo "query_cache_type=0 "  >> /etc/my.cnf
echo "query_cache_size = 0 "  >> /etc/my.cnf
echo "query_cache_strip_comments = 1 "  >> /etc/my.cnf
echo "bulk_insert_buffer_size = 32M "  >> /etc/my.cnf
echo "thread_cache_size = 1024 "  >> /etc/my.cnf
echo "thread_stack = 512K "  >> /etc/my.cnf
echo "#--- InnoDB ---# "  >> /etc/my.cnf
echo "innodb_data_home_dir = ${dataDir}/data "  >> /etc/my.cnf
echo "innodb_data_file_path=ibdata1:10M:autoextend "  >> /etc/my.cnf
echo "innodb_buffer_pool_size = 512M "  >> /etc/my.cnf
echo "innodb_buffer_pool_instances = 21 "  >> /etc/my.cnf
echo "innodb_log_file_size = 1024M "  >> /etc/my.cnf
echo "innodb_log_buffer_size = 64M "  >> /etc/my.cnf
echo "innodb_log_files_in_group = 3 "  >> /etc/my.cnf
echo "innodb_flush_log_at_trx_commit = 1 "  >> /etc/my.cnf
echo "innodb_lock_wait_timeout = 20 "  >> /etc/my.cnf
echo "innodb_sync_spin_loops = 40 "  >> /etc/my.cnf
echo "innodb_max_dirty_pages_pct = 75 "  >> /etc/my.cnf
echo "innodb_thread_concurrency = 0 "  >> /etc/my.cnf
echo "log_bin_trust_function_creators = 1 "  >> /etc/my.cnf
echo "innodb_flush_method = O_DIRECT "  >> /etc/my.cnf
echo "innodb_file_per_table = 1 "  >> /etc/my.cnf
echo "innodb_read_io_threads = 16 "  >> /etc/my.cnf
echo "innodb_write_io_threads = 16 "  >> /etc/my.cnf
echo "innodb_io_capacity = 2000 "  >> /etc/my.cnf
echo "innodb_purge_threads=4 "  >> /etc/my.cnf
echo "innodb_purge_batch_size = 300 "  >> /etc/my.cnf
echo "innodb_change_buffering=all "  >> /etc/my.cnf
echo "innodb_stats_on_metadata=0 "  >> /etc/my.cnf
echo "#---new---# "  >> /etc/my.cnf
echo "eq_range_index_dive_limit = 10000 "  >> /etc/my.cnf
echo "auto_increment_increment=1 "  >> /etc/my.cnf
echo "auto_increment_offset=1 "  >> /etc/my.cnf
echo "[mysqldump] "  >> /etc/my.cnf
echo "quick "  >> /etc/my.cnf
echo "max_allowed_packet = 1024M "  >> /etc/my.cnf
echo "myisam_max_sort_file_size = 10G "  >> /etc/my.cnf
echo "[mysql] "  >> /etc/my.cnf
echo "no-auto-rehash "  >> /etc/my.cnf
echo "[myisamchk] "  >> /etc/my.cnf
echo "key_buffer_size = 64M "  >> /etc/my.cnf
echo "sort_buffer_size = 256k "  >> /etc/my.cnf
echo "read_buffer = 2M "  >> /etc/my.cnf
echo "write_buffer = 2M "  >> /etc/my.cnf
echo "[mysqlhotcopy] "  >> /etc/my.cnf
echo "interactive-timeout "  >> /etc/my.cnf
echo "[mysqld_safe] "  >> /etc/my.cnf
echo "open-files-limit = 68192 "  >> /etc/my.cnf
echo -e "\033[32m ***********************SUCCESS**************************** \033[0m"
echo -e "\n"

echo "********************5.初始化数据库************************"
/usr/local/mysql/scripts/mysql_install_db --defaults-file=/etc/my.cnf --user=mysql > /tmp/mysql_install.log
if [ `cat /tmp/mysql_install.log | sed -n '2p'` != "OK" ]; then
  echo -e "\033[31m MariaDB安装失败，请检查文件 /tmp/mysql_install.log 和 $dataDir/log/error.log 中输出的错误信息 \033[0m"
  exit 0
fi
echo -e "\033[32m ***********************SUCCESS**************************** \033[0m"
echo -e "\n"

echo "********************6.配置自启动服务**********************"
cp /usr/local/mysql/support-files/systemd/mariadb.service /usr/lib/systemd/system/mariadb.service
if [ `grep -c "/usr/local/mysql/bin" /etc/profile` -eq 0 ]; then
  echo "export PATH=/usr/local/mysql/bin:$PATH" >> /etc/profile
fi
export PATH=/usr/local/mysql/bin:$PATH
cp -f /usr/local/mysql/bin/mysql /usr/local/bin
systemctl daemon-reload
systemctl enable mariadb
systemctl stop mariadb
systemctl start mariadb

if [ `systemctl status mariadb | grep running | grep -v grep | wc -l` -gt 0 ]; then  
  echo -e "\033[32m ***********************SUCCESS**************************** \033[0m"
else
  echo -e "\033[31m MariaDB启动异常，清检查 $dataDir/log/error.log 中输出的错误信息 \033[0m"
fi

echo -e "\n"

echo "********************7.创建数据库与用户**************************"
while :
do
  echo -e "\033[45;37m 请输入数据库名称，不要包含特殊字符 \033[0m"
  read dbName
  if [ -z "${dbName}" ] || [[ ${dbName} =~ [[:space:]] ]] || [[ ${dbName} =~ [^a-zA-Z0-9] ]]; then
    echo -e "\033[31m 输入的值为空或包含特殊字符,请重新输入 \033[0m"
  else 
    break
  fi
done

echo -e "\n"
while :
do
  echo -e "\033[45;37m 请输入访问 ${dbName} 数据库的用户名,仅字母与数字 \033[0m"
  read dbUser
  if [ -z "${dbUser}" ] || [[ ${dbUser} =~ [[:space:]] ]] || [[ ${dbUser} =~ [^a-zA-Z0-9] ]]; then
    echo -e "\033[31m 输入的值为空或包含特殊字符,请重新输入 \033[0m"
  else
    break
  fi
done

echo -e "\n" 
while true; do
    echo -e "\033[45;37m 密码必须包含2个大写字母与2个数字、不少于8位并且至少有一位非字母与数字的字符(空格除外) 、并且忽略首尾空格\033[0m"
    echo -e " 请输入密码："
    #read -s -p "Enter Password: " dbPasswd  #不显示输入的字符
    read  dbPasswd
    echo -e "\033[45;37m 您输入的密码为：$dbPasswd \033[0m"
    echo
    FAIL=no
    # Mininum 8 characters
    [[ ${#dbPasswd} -ge 8 ]] || FAIL=yes

    # Must contain 2 upper case letters
    echo $dbPasswd | grep -q "[A-Z].*[A-Z]" || FAIL=yes

    # Must contain 2 digits
    echo $dbPasswd | grep -q "[0-9].*[0-9]" || FAIL=yes

    # Must contain one non-alphanumeric character (no space)
    echo $dbPasswd | grep -q "[^a-zA-Z0-9 ]" || FAIL=yes

    [[ ${FAIL} == "no" ]] && [[ ! ${dbPasswd} =~ [[:space:]] ]] && break
done

dbCharacter="utf8mb4"
mysql -e "create database ${dbName} default character set ${dbCharacter} collate utf8mb4_bin; " 1>&2 || exit
mysql -e "create user ${dbUser}@'%' identified by '${dbPasswd}'; " 1>&2 || exit
mysql -e "create user ${dbUser}@'localhost' identified by '${dbPasswd}'; " 1>&2 || exit
mysql -e "grant all privileges on ${dbName}.* to ${dbUser}@'%' with grant option; " 1>&2 || exit
mysql -e "grant all privileges on ${dbName}.* to ${dbUser}@'localhost' with grant option; " 1>&2 || exit
mysql -e "flush privileges;" 1>&2 || exit
echo -e "\033[32m ***********************SUCCESS**************************** \033[0m"
echo -e "\n"

echo -e "\033[45;37m **********MariaDB数据库配置如下：********************** \033[0m"
echo "数据库服务器IP地址：`ip a|grep inet|grep -v 127.0.0.1|grep -v inet6|awk '{print $2}'|tr -d "addr:"` "
echo "数据库服务器端口：6006 "
echo "新创建的数据库名称：${dbName} "
echo "新创建的数据库字符集：${dbCharacter} "
echo "新创建的数据库用户：${dbUser} "
echo "新创建的数据库密码：${dbPasswd} "
echo "查看 MariaDB 运行是否正常使用命令：systemctl status mariadb "
echo "停止 MariaDB 服务使用的命令：systemctl stop mariadb "
echo "启动 MariaDB 服务使用的命令：systemctl start mariadb "
echo "启动日志位于：$dataDir/log/error.log "
echo "数据文件位于：$dataDir/data/ 目录"
echo "慢SQL日志位于：${dataDir}/log/mysql-slow.log"
echo -e "\033[32m ***********************MariaDB数据库创建成功*************** \033[0m"
echo -e "\n"

